package kr.co.vividnext.sodalive.explorer.profile.creatorCommunity

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.QCreatorCommunity.creatorCommunity
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository

interface CreatorCommunityRepository : JpaRepository<CreatorCommunity, Long>, CreatorCommunityQueryRepository

interface CreatorCommunityQueryRepository {
    fun findByIdAndMemberId(id: Long, memberId: Long): CreatorCommunity?

    fun getCommunityPostList(
        creatorId: Long,
        offset: Long,
        limit: Long,
        isAdult: Boolean
    ): List<CreatorCommunity>

    fun findByIdAndActive(postId: Long): CreatorCommunity?
}

class CreatorCommunityQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : CreatorCommunityQueryRepository {
    override fun findByIdAndMemberId(id: Long, memberId: Long): CreatorCommunity? {
        return queryFactory
            .selectFrom(creatorCommunity)
            .where(
                creatorCommunity.id.eq(id)
                    .and(creatorCommunity.member.id.eq(memberId))
            )
            .fetchFirst()
    }

    override fun getCommunityPostList(
        creatorId: Long,
        offset: Long,
        limit: Long,
        isAdult: Boolean
    ): List<CreatorCommunity> {
        var where = creatorCommunity.member.id.eq(creatorId)
            .and(creatorCommunity.isActive.isTrue)

        if (!isAdult) {
            where = where.and(creatorCommunity.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(creatorCommunity)
            .innerJoin(creatorCommunity.member, member)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(creatorCommunity.createdAt.desc())
            .fetch()
    }

    override fun findByIdAndActive(postId: Long): CreatorCommunity? {
        return queryFactory
            .selectFrom(creatorCommunity)
            .where(
                creatorCommunity.id.eq(postId)
                    .and(creatorCommunity.isActive.isTrue)
            )
            .fetchFirst()
    }
}
